"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NttAutomaticRoute = exports.nttAutomaticRoute = void 0;
const sdk_connect_1 = require("@wormhole-foundation/sdk-connect");
require("@wormhole-foundation/sdk-definitions-ntt");
const types_js_1 = require("./types.js");
function nttAutomaticRoute(config) {
    class NttRouteImpl extends NttAutomaticRoute {
        static config = config;
    }
    return NttRouteImpl;
}
exports.nttAutomaticRoute = nttAutomaticRoute;
class NttAutomaticRoute extends sdk_connect_1.routes.AutomaticRoute {
    NATIVE_GAS_DROPOFF_SUPPORTED = true;
    IS_AUTOMATIC = true;
    // @ts-ignore
    // Since we set the config on the static class, access it with this param
    // the NttManualRoute.config will always be empty
    staticConfig = this.constructor.config;
    static config = { tokens: {} };
    static meta = { name: "AutomaticNtt" };
    static supportedNetworks() {
        return types_js_1.NttRoute.resolveSupportedNetworks(this.config);
    }
    static supportedChains(network) {
        return types_js_1.NttRoute.resolveSupportedChains(this.config, network);
    }
    static async supportedSourceTokens(fromChain) {
        return types_js_1.NttRoute.resolveSourceTokens(this.config, fromChain);
    }
    static async supportedDestinationTokens(sourceToken, fromChain, toChain) {
        return types_js_1.NttRoute.resolveDestinationTokens(this.config, sourceToken, fromChain, toChain);
    }
    static isProtocolSupported(chain) {
        return chain.supportsProtocol("Ntt");
    }
    getDefaultOptions() {
        return types_js_1.NttRoute.AutomaticOptions;
    }
    async isAvailable() {
        // TODO: check that both src/dst are available for relayed NTT transfers
        return true;
        //throw new Error("Method not implemented.");
    }
    async validate(params) {
        const options = params.options ?? this.getDefaultOptions();
        const gasDropoff = sdk_connect_1.amount.parse(options.gasDropoff ?? "0.0", this.request.toChain.config.nativeTokenDecimals);
        const amt = sdk_connect_1.amount.parse(params.amount, this.request.source.decimals);
        const validatedParams = {
            amount: params.amount,
            normalizedParams: {
                amount: amt,
                sourceContracts: types_js_1.NttRoute.resolveNttContracts(this.staticConfig, this.request.source.id),
                destinationContracts: types_js_1.NttRoute.resolveNttContracts(this.staticConfig, this.request.destination.id),
                options: {
                    queue: false,
                    automatic: true,
                    gasDropoff: sdk_connect_1.amount.units(gasDropoff),
                },
            },
            options,
        };
        return { valid: true, params: validatedParams };
    }
    async quote(params) {
        const { fromChain, toChain } = this.request;
        const ntt = await fromChain.getProtocol("Ntt", {
            ntt: params.normalizedParams.sourceContracts,
        });
        const deliveryPrice = await ntt.quoteDeliveryPrice(toChain.chain, params.normalizedParams.options);
        return {
            success: true,
            params,
            sourceToken: {
                token: this.request.source.id,
                amount: params.normalizedParams.amount,
            },
            destinationToken: {
                token: this.request.destination.id,
                amount: sdk_connect_1.amount.parse(params.amount, this.request.destination.decimals),
            },
            relayFee: {
                token: sdk_connect_1.Wormhole.tokenId(fromChain.chain, "native"),
                amount: sdk_connect_1.amount.fromBaseUnits(deliveryPrice, fromChain.config.nativeTokenDecimals),
            },
            destinationNativeGas: sdk_connect_1.amount.fromBaseUnits(params.normalizedParams.options.gasDropoff ?? 0n, toChain.config.nativeTokenDecimals),
        };
    }
    async initiate(signer, quote) {
        const { params } = quote;
        const { fromChain, from, to } = this.request;
        const sender = sdk_connect_1.Wormhole.parseAddress(signer.chain(), signer.address());
        const ntt = await fromChain.getProtocol("Ntt", {
            ntt: params.normalizedParams.sourceContracts,
        });
        const initXfer = ntt.transfer(sender, sdk_connect_1.amount.units(params.normalizedParams.amount), to, params.normalizedParams.options);
        const txids = await (0, sdk_connect_1.signSendWait)(fromChain, initXfer, signer);
        return {
            from: from.chain,
            to: to.chain,
            state: sdk_connect_1.TransferState.SourceInitiated,
            originTxs: txids,
            params,
        };
    }
    async *track(receipt, timeout) {
        if ((0, sdk_connect_1.isSourceInitiated)(receipt) || (0, sdk_connect_1.isSourceFinalized)(receipt)) {
            const { txid } = receipt.originTxs[receipt.originTxs.length - 1];
            const vaa = await this.wh.getVaa(txid, "Ntt:WormholeTransfer", timeout);
            if (!vaa)
                throw new Error("No VAA found for transaction: " + txid);
            const msgId = {
                chain: vaa.emitterChain,
                emitter: vaa.emitterAddress,
                sequence: vaa.sequence,
            };
            receipt = {
                ...receipt,
                state: sdk_connect_1.TransferState.Attested,
                attestation: {
                    id: msgId,
                    attestation: vaa,
                },
            };
            yield receipt;
        }
        const { toChain } = this.request;
        const ntt = await toChain.getProtocol("Ntt", {
            ntt: receipt.params.normalizedParams.destinationContracts,
        });
        if ((0, sdk_connect_1.isAttested)(receipt)) {
            const { attestation: { attestation: vaa }, } = receipt;
            if (await ntt.getIsApproved(vaa)) {
                receipt = {
                    ...receipt,
                    state: sdk_connect_1.TransferState.DestinationInitiated,
                    // TODO: check for destination event transactions to get dest Txids
                };
                yield receipt;
            }
        }
        if ((0, sdk_connect_1.isRedeemed)(receipt)) {
            const { attestation: { attestation: vaa }, } = receipt;
            if (await ntt.getIsExecuted(vaa)) {
                receipt = {
                    ...receipt,
                    state: sdk_connect_1.TransferState.DestinationFinalized,
                };
                yield receipt;
            }
        }
        yield receipt;
    }
}
exports.NttAutomaticRoute = NttAutomaticRoute;
//# sourceMappingURL=automatic.js.map